package java.util;

import java.util.function.Consumer;
import java.lang.reflect.Array;
import java.util.function.BiFunction;
import java.util.function.BiConsumer;
import java.io.StreamCorruptedException;
import sun.misc.SharedSecrets;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.Serializable;

public class IdentityHashMap<K, V> extends AbstractMap<K, V> implements Map<K, V>, Serializable, Cloneable
{
    private static final int DEFAULT_CAPACITY = 32;
    private static final int MINIMUM_CAPACITY = 4;
    private static final int MAXIMUM_CAPACITY = 536870912;
    transient Object[] table;
    int size;
    transient int modCount;
    static final Object NULL_KEY;
    private transient Set<Map.Entry<K, V>> entrySet;
    private static final long serialVersionUID = 8188218128353913216L;
    
    private static Object maskNull(Object o) {
        return (o == null) ? IdentityHashMap.NULL_KEY : o;
    }
    
    static final Object unmaskNull(Object o) {
        return (o == IdentityHashMap.NULL_KEY) ? null : o;
    }
    
    public IdentityHashMap() {
        this.init(32);
    }
    
    public IdentityHashMap(int i) {
        if (i < 0) {
            throw new IllegalArgumentException("expectedMaxSize is negative: " + i);
        }
        this.init(capacity(i));
        return;
    }
    
    private static int capacity(int n) {
        return (n > 178956970) ? 536870912 : ((n <= 2) ? 4 : Integer.highestOneBit(n + (n << 1)));
    }
    
    private void init(int n) {
        this.table = new Object[2 * n];
    }
    
    public IdentityHashMap(Map<? extends K, ? extends V> map) {
        this((int)((double)(1 + map.size()) * 1.1));
        this.putAll(map);
    }
    
    @Override
    public int size() {
        return this.size;
    }
    
    @Override
    public boolean isEmpty() {
        return this.size == 0;
    }
    
    private static int hash(Object o, int n) {
        int identityHashCode = System.identityHashCode(o);
        return (identityHashCode << 1) - (identityHashCode << 8) & n - 1;
    }
    
    private static int nextKeyIndex(int n, int n2) {
        return (n + 2 < n2) ? (n + 2) : 0;
    }
    
    @Override
    public V get(Object o) {
        Object maskNull = maskNull(o);
        Object[] table = this.table;
        int length = table.length;
        int n = hash(maskNull, length);
        while (true) {
            Object o2 = table[n];
            if (o2 == maskNull) {
                return table[n + 1];
            }
            if (o2 == null) {
                return null;
            }
            n = nextKeyIndex(n, length);
        }
    }
    
    @Override
    public boolean containsKey(Object o) {
        Object maskNull = maskNull(o);
        Object[] table = this.table;
        int length = table.length;
        int n = hash(maskNull, length);
        while (true) {
            Object o2 = table[n];
            if (o2 == maskNull) {
                return true;
            }
            if (o2 == null) {
                return false;
            }
            n = nextKeyIndex(n, length);
        }
    }
    
    @Override
    public boolean containsValue(Object o) {
        Object[] table = this.table;
        int i = 1;
        while (i < table.length) {
            if (table[i] == o && table[i - 1] != null) {
                return true;
            }
            i += 2;
        }
        return false;
    }
    
    private boolean containsMapping(Object o, Object o2) {
        Object maskNull = maskNull(o);
        Object[] table = this.table;
        int length = table.length;
        int n = hash(maskNull, length);
        while (true) {
            Object o3 = table[n];
            if (o3 == maskNull) {
                return table[n + 1] == o2;
            }
            if (o3 == null) {
                return false;
            }
            n = nextKeyIndex(n, length);
        }
    }
    
    @Override
    public V put(K k, V v) {
        Object maskNull = maskNull(k);
        while (true) {
            Object[] table = this.table;
            int length = table.length;
            int n = hash(maskNull, length);
            Object o;
            while ((o = table[n]) != null) {
                if (o == maskNull) {
                    Object o2 = table[n + 1];
                    table[n + 1] = v;
                    return o2;
                }
                n = nextKeyIndex(n, length);
            }
            int size = this.size + 1;
            if (size + (size << 1) > length && this.resize(length)) {
                continue;
            }
            ++this.modCount;
            table[n] = maskNull;
            table[n + 1] = v;
            this.size = size;
            return null;
        }
    }
    
    private boolean resize(int n) {
        int n2 = n * 2;
        Object[] table = this.table;
        int length = table.length;
        if (length == 1073741824) {
            if (this.size == 536870911) {
                throw new IllegalStateException("Capacity exhausted.");
            }
            return false;
        }
        else {
            if (length >= n2) {
                return false;
            }
            Object[] table2 = new Object[n2];
            for (int i = 0; i < length; i += 2) {
                Object o = table[i];
                if (o != null) {
                    Object o2 = table[i + 1];
                    table[i + 1] = (table[i] = null);
                    int n3;
                    for (n3 = hash(o, n2); table2[n3] != null; n3 = nextKeyIndex(n3, n2)) {}
                    table2[n3] = o;
                    table2[n3 + 1] = o2;
                }
            }
            this.table = table2;
            return true;
        }
    }
    
    @Override
    public void putAll(Map<? extends K, ? extends V> map) {
        int size = map.size();
        if (size == 0) {
            return;
        }
        if (size > this.size) {
            this.resize(capacity(size));
        }
        for (Map.Entry entry : map.entrySet()) {
            this.put(entry.getKey(), entry.getValue());
        }
        return;
    }
    
    @Override
    public V remove(Object o) {
        Object maskNull = maskNull(o);
        Object[] table = this.table;
        int length = table.length;
        int n = hash(maskNull, length);
        while (true) {
            Object o2 = table[n];
            if (o2 == maskNull) {
                ++this.modCount;
                --this.size;
                Object o3 = table[n + 1];
                table[n] = (table[n + 1] = null);
                this.closeDeletion(n);
                return o3;
            }
            if (o2 == null) {
                return null;
            }
            n = nextKeyIndex(n, length);
        }
    }
    
    private boolean removeMapping(Object o, Object o2) {
        Object maskNull = maskNull(o);
        Object[] table = this.table;
        int length = table.length;
        int n = hash(maskNull, length);
        while (true) {
            Object o3 = table[n];
            if (o3 == maskNull) {
                if (table[n + 1] != o2) {
                    return false;
                }
                ++this.modCount;
                --this.size;
                table[n + 1] = (table[n] = null);
                this.closeDeletion(n);
                return true;
            }
            else {
                if (o3 == null) {
                    return false;
                }
                n = nextKeyIndex(n, length);
            }
        }
    }
    
    private void closeDeletion(int n) {
        Object[] table = this.table;
        Object o;
        for (int length = table.length, n2 = nextKeyIndex(n, length); (o = table[n2]) != null; n2 = nextKeyIndex(n2, length)) {
            int hash = hash(o, length);
            if ((n2 < hash && (hash <= n || n <= n2)) || (hash <= n && n <= n2)) {
                table[n] = o;
                table[n + 1] = table[n2 + 1];
                table[n2 + 1] = (table[n2] = null);
                n = n2;
            }
        }
    }
    
    @Override
    public void clear() {
        ++this.modCount;
        Object[] table = this.table;
        for (int i = 0; i < table.length; ++i) {
            table[i] = null;
        }
        this.size = 0;
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if ((o instanceof IdentityHashMap)) {
            IdentityHashMap<Object, Object> identityHashMap = (IdentityHashMap)o;
            if (identityHashMap.size() != this.size) {
                return false;
            }
            Object[] table = identityHashMap.table;
            int i = 0;
            while (i < table.length) {
                Object o2 = table[i];
                if (o2 != null && !this.containsMapping(o2, table[i + 1])) {
                    return false;
                }
                i += 2;
            }
            return true;
        }
        else {
            return o instanceof Map && this.entrySet().equals(((Map)o).entrySet());
        }
    }
    
    @Override
    public int hashCode() {
        int n = 0;
        Object[] table = this.table;
        for (int i = 0; i < table.length; i += 2) {
            Object o = table[i];
            if (o != null) {
                n += (System.identityHashCode(unmaskNull(o)) ^ System.identityHashCode(table[i + 1]));
            }
        }
        return n;
    }
    
    public Object clone() {
        try {
            IdentityHashMap<Object, Object> identityHashMap = (IdentityHashMap)super.clone();
            identityHashMap.entrySet = null;
            identityHashMap.table = (Object[])this.table.clone();
            return identityHashMap;
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause);
        }
    }
    
    @Override
    public Set<K> keySet() {
        Set<Object> keySet = this.keySet;
        if (keySet == null) {
            keySet = new IdentityHashMap.KeySet();
            this.keySet = keySet;
            return keySet;
        }
        return keySet;
    }
    
    @Override
    public Collection<V> values() {
        Collection<Object> values = this.values;
        if (values == null) {
            values = new IdentityHashMap.Values();
            this.values = values;
            return values;
        }
        return values;
    }
    
    @Override
    public Set<Map.Entry<K, V>> entrySet() {
        Set<Object> entrySet = this.entrySet;
        if (entrySet != null) {
            return entrySet;
        }
        return this.entrySet = new IdentityHashMap.EntrySet();
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        objectOutputStream.writeInt(this.size);
        Object[] table = this.table;
        for (int i = 0; i < table.length; i += 2) {
            Object o = table[i];
            if (o != null) {
                objectOutputStream.writeObject(unmaskNull(o));
                objectOutputStream.writeObject(table[i + 1]);
            }
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        int int1 = objectInputStream.readInt();
        if (int1 < 0) {
            throw new StreamCorruptedException("Illegal mappings count: " + int1);
        }
        int capacity = capacity(int1);
        SharedSecrets.getJavaOISAccess().checkArray(objectInputStream, Object[].class, capacity);
        this.init(capacity);
        for (int i = 0; i < int1; ++i) {
            this.putForCreate(objectInputStream.readObject(), objectInputStream.readObject());
        }
        return;
    }
    
    private void putForCreate(K k, V v) throws StreamCorruptedException {
        Object maskNull = maskNull(k);
        Object[] table = this.table;
        int length = table.length;
        int n = hash(maskNull, length);
        Object o;
        while ((o = table[n]) != null) {
            if (o == maskNull) {
                throw new StreamCorruptedException();
            }
            n = nextKeyIndex(n, length);
        }
        table[n] = maskNull;
        table[n + 1] = v;
    }
    
    @Override
    public void forEach(BiConsumer<? super K, ? super V> biConsumer) {
        Objects.requireNonNull(biConsumer);
        int modCount = this.modCount;
        Object[] table = this.table;
        int i = 0;
        while (i < table.length) {
            Object o = table[i];
            if (o != null) {
                biConsumer.accept(unmaskNull(o), table[i + 1]);
            }
            if (this.modCount != modCount) {
                throw new ConcurrentModificationException();
            }
            i += 2;
        }
    }
    
    @Override
    public void replaceAll(BiFunction<? super K, ? super V, ? extends V> biFunction) {
        Objects.requireNonNull(biFunction);
        int modCount = this.modCount;
        Object[] table = this.table;
        int i = 0;
        while (i < table.length) {
            Object o = table[i];
            if (o != null) {
                table[i + 1] = biFunction.apply(unmaskNull(o), table[i + 1]);
            }
            if (this.modCount != modCount) {
                throw new ConcurrentModificationException();
            }
            i += 2;
        }
    }
    
    static int access$000(int n, int n2) {
        return nextKeyIndex(n, n2);
    }
    
    static int access$100(Object o, int n) {
        return hash(o, n);
    }
    
    static boolean access$1100(IdentityHashMap identityHashMap, Object o, Object o2) {
        return identityHashMap.containsMapping(o, o2);
    }
    
    static boolean access$1200(IdentityHashMap identityHashMap, Object o, Object o2) {
        return identityHashMap.removeMapping(o, o2);
    }
    
    static {
        NULL_KEY = new Object();
    }
    
    private abstract class IdentityHashMapIterator<T> implements Iterator<T>
    {
        int index;
        int expectedModCount;
        int lastReturnedIndex;
        boolean indexValid;
        Object[] traversalTable;
        
        private IdentityHashMapIterator() {
            this.index = ((IdentityHashMap.this.size != 0) ? 0 : IdentityHashMap.this.table.length);
            this.expectedModCount = IdentityHashMap.this.modCount;
            this.lastReturnedIndex = -1;
            this.traversalTable = IdentityHashMap.this.table;
        }
        
        @Override
        public boolean hasNext() {
            Object[] traversalTable = this.traversalTable;
            int i = this.index;
            while (i < traversalTable.length) {
                if (traversalTable[i] != null) {
                    this.index = i;
                    return this.indexValid = true;
                }
                i += 2;
            }
            this.index = traversalTable.length;
            return false;
        }
        
        protected int nextIndex() {
            if (IdentityHashMap.this.modCount != this.expectedModCount) {
                throw new ConcurrentModificationException();
            }
            if (!this.indexValid && !this.hasNext()) {
                throw new NoSuchElementException();
            }
            this.indexValid = false;
            this.lastReturnedIndex = this.index;
            this.index += 2;
            return this.lastReturnedIndex;
        }
        
        @Override
        public void remove() {
            if (this.lastReturnedIndex == -1) {
                throw new IllegalStateException();
            }
            if (IdentityHashMap.this.modCount != this.expectedModCount) {
                throw new ConcurrentModificationException();
            }
            this.expectedModCount = ++IdentityHashMap.this.modCount;
            int lastReturnedIndex = this.lastReturnedIndex;
            this.lastReturnedIndex = -1;
            this.index = lastReturnedIndex;
            this.indexValid = false;
            Object[] traversalTable = this.traversalTable;
            int length = traversalTable.length;
            Object o = traversalTable[lastReturnedIndex];
            traversalTable[lastReturnedIndex + 1] = (traversalTable[lastReturnedIndex] = null);
            if (traversalTable != IdentityHashMap.this.table) {
                IdentityHashMap.this.remove(o);
                this.expectedModCount = IdentityHashMap.this.modCount;
                return;
            }
            IdentityHashMap<Object, Object> this$0 = IdentityHashMap.this;
            --this$0.size;
            Object o2;
            for (int n = IdentityHashMap.access$000(lastReturnedIndex, length); (o2 = traversalTable[n]) != null; n = IdentityHashMap.access$000(n, length)) {
                int access$100 = IdentityHashMap.access$100(o2, length);
                if ((n < access$100 && (access$100 <= lastReturnedIndex || lastReturnedIndex <= n)) || (access$100 <= lastReturnedIndex && lastReturnedIndex <= n)) {
                    if (n < lastReturnedIndex && lastReturnedIndex >= lastReturnedIndex && this.traversalTable == IdentityHashMap.this.table) {
                        int n2 = length - lastReturnedIndex;
                        Object[] traversalTable2 = new Object[n2];
                        System.arraycopy(traversalTable, lastReturnedIndex, traversalTable2, 0, n2);
                        this.traversalTable = traversalTable2;
                        this.index = 0;
                    }
                    traversalTable[lastReturnedIndex] = o2;
                    traversalTable[lastReturnedIndex + 1] = traversalTable[n + 1];
                    traversalTable[n + 1] = (traversalTable[n] = null);
                    lastReturnedIndex = n;
                }
            }
            return;
        }
    }
    
    private class KeyIterator extends IdentityHashMap.IdentityHashMapIterator<K>
    {
        @Override
        public K next() {
            return IdentityHashMap.unmaskNull(this.traversalTable[this.nextIndex()]);
        }
    }
    
    private class ValueIterator extends IdentityHashMap.IdentityHashMapIterator<V>
    {
        @Override
        public V next() {
            return this.traversalTable[this.nextIndex() + 1];
        }
    }
    
    private class EntryIterator extends IdentityHashMap.IdentityHashMapIterator<Map.Entry<K, V>>
    {
        private IdentityHashMap.EntryIterator.Entry lastReturnedEntry;
        
        @Override
        public Map.Entry<K, V> next() {
            return this.lastReturnedEntry = new IdentityHashMap.EntryIterator.Entry(this.nextIndex());
        }
        
        @Override
        public void remove() {
            this.lastReturnedIndex = ((null == this.lastReturnedEntry) ? -1 : this.lastReturnedEntry.index);
            super.remove();
            this.lastReturnedEntry.index = this.lastReturnedIndex;
            this.lastReturnedEntry = null;
        }
        
        private class Entry implements Map.Entry<K, V>
        {
            private int index;
            
            private Entry(int index) {
                this.index = index;
            }
            
            @Override
            public K getKey() {
                this.checkIndexForEntryUse();
                return IdentityHashMap.unmaskNull(IdentityHashMap.EntryIterator.this.traversalTable[this.index]);
            }
            
            @Override
            public V getValue() {
                this.checkIndexForEntryUse();
                return IdentityHashMap.EntryIterator.this.traversalTable[this.index + 1];
            }
            
            @Override
            public V setValue(V v) {
                this.checkIndexForEntryUse();
                Object o = IdentityHashMap.EntryIterator.this.traversalTable[this.index + 1];
                IdentityHashMap.EntryIterator.this.traversalTable[this.index + 1] = v;
                if (IdentityHashMap.EntryIterator.this.traversalTable != IdentityHashMap.this.table) {
                    IdentityHashMap.this.put(IdentityHashMap.EntryIterator.this.traversalTable[this.index], v);
                    return o;
                }
                return o;
            }
            
            @Override
            public boolean equals(Object obj) {
                if (this.index < 0) {
                    return super.equals(obj);
                }
                if (!(obj instanceof Map.Entry)) {
                    return false;
                }
                Map.Entry<Object, Object> entry = (Map.Entry)obj;
                return entry.getKey() == IdentityHashMap.unmaskNull(IdentityHashMap.EntryIterator.this.traversalTable[this.index]) && entry.getValue() == IdentityHashMap.EntryIterator.this.traversalTable[this.index + 1];
            }
            
            @Override
            public int hashCode() {
                if (IdentityHashMap.EntryIterator.this.lastReturnedIndex < 0) {
                    return super.hashCode();
                }
                return System.identityHashCode(IdentityHashMap.unmaskNull(IdentityHashMap.EntryIterator.this.traversalTable[this.index])) ^ System.identityHashCode(IdentityHashMap.EntryIterator.this.traversalTable[this.index + 1]);
            }
            
            @Override
            public String toString() {
                if (this.index < 0) {
                    return super.toString();
                }
                return IdentityHashMap.unmaskNull(IdentityHashMap.EntryIterator.this.traversalTable[this.index]) + "=" + IdentityHashMap.EntryIterator.this.traversalTable[this.index + 1];
            }
            
            private void checkIndexForEntryUse() {
                if (this.index < 0) {
                    throw new IllegalStateException("Entry was removed");
                }
                return;
            }
        }
    }
    
    private class KeySet extends AbstractSet<K>
    {
        @Override
        public Iterator<K> iterator() {
            return new IdentityHashMap.KeyIterator();
        }
        
        @Override
        public int size() {
            return IdentityHashMap.this.size;
        }
        
        @Override
        public boolean contains(Object o) {
            return IdentityHashMap.this.containsKey(o);
        }
        
        @Override
        public boolean remove(Object o) {
            int size = IdentityHashMap.this.size;
            IdentityHashMap.this.remove(o);
            return IdentityHashMap.this.size != size;
        }
        
        @Override
        public boolean removeAll(Collection<?> collection) {
            Objects.requireNonNull(collection);
            boolean n = false;
            Iterator<K> iterator = this.iterator();
            while (iterator.hasNext()) {
                if (!collection.contains(iterator.next())) {
                    continue;
                }
                iterator.remove();
                n = true;
            }
            return n;
        }
        
        @Override
        public void clear() {
            IdentityHashMap.this.clear();
        }
        
        @Override
        public int hashCode() {
            int n = 0;
            Iterator<K> iterator = this.iterator();
            while (iterator.hasNext()) {
                n += System.identityHashCode(iterator.next());
            }
            return n;
        }
        
        @Override
        public Object[] toArray() {
            return this.toArray(new Object[0]);
        }
        
        @Override
        public <T> T[] toArray(T[] array) {
            int modCount = IdentityHashMap.this.modCount;
            int size = this.size();
            if (array.length < size) {
                array = (Object[])(Object[])Array.newInstance(array.getClass().getComponentType(), size);
            }
            Object[] table = IdentityHashMap.this.table;
            int n = 0;
            for (int i = 0; i < table.length; i += 2) {
                Object o;
                if ((o = table[i]) != null) {
                    if (n >= size) {
                        throw new ConcurrentModificationException();
                    }
                    array[n++] = IdentityHashMap.unmaskNull(o);
                }
            }
            if (n < size || modCount != IdentityHashMap.this.modCount) {
                throw new ConcurrentModificationException();
            }
            if (n < array.length) {
                array[n] = null;
                return array;
            }
            return array;
        }
        
        @Override
        public Spliterator<K> spliterator() {
            return new IdentityHashMap.KeySpliterator(IdentityHashMap.this, 0, -1, 0, 0);
        }
    }
    
    private class Values extends AbstractCollection<V>
    {
        @Override
        public Iterator<V> iterator() {
            return new IdentityHashMap.ValueIterator();
        }
        
        @Override
        public int size() {
            return IdentityHashMap.this.size;
        }
        
        @Override
        public boolean contains(Object o) {
            return IdentityHashMap.this.containsValue(o);
        }
        
        @Override
        public boolean remove(Object o) {
            Iterator<V> iterator = this.iterator();
            while (iterator.hasNext()) {
                if (iterator.next() != o) {
                    continue;
                }
                iterator.remove();
                return true;
            }
            return false;
        }
        
        @Override
        public void clear() {
            IdentityHashMap.this.clear();
        }
        
        @Override
        public Object[] toArray() {
            return this.toArray(new Object[0]);
        }
        
        @Override
        public <T> T[] toArray(T[] array) {
            int modCount = IdentityHashMap.this.modCount;
            int size = this.size();
            if (array.length < size) {
                array = (Object[])(Object[])Array.newInstance(array.getClass().getComponentType(), size);
            }
            Object[] table = IdentityHashMap.this.table;
            int n = 0;
            for (int i = 0; i < table.length; i += 2) {
                if (table[i] != null) {
                    if (n >= size) {
                        throw new ConcurrentModificationException();
                    }
                    array[n++] = table[i + 1];
                }
            }
            if (n < size || modCount != IdentityHashMap.this.modCount) {
                throw new ConcurrentModificationException();
            }
            if (n < array.length) {
                array[n] = null;
                return array;
            }
            return array;
        }
        
        @Override
        public Spliterator<V> spliterator() {
            return new IdentityHashMap.ValueSpliterator(IdentityHashMap.this, 0, -1, 0, 0);
        }
    }
    
    private class EntrySet extends AbstractSet<Map.Entry<K, V>>
    {
        @Override
        public Iterator<Map.Entry<K, V>> iterator() {
            return new IdentityHashMap.EntryIterator();
        }
        
        @Override
        public boolean contains(Object o) {
            if (!(o instanceof Map.Entry)) {
                return false;
            }
            Map.Entry<Object, Object> entry = (Map.Entry)o;
            return IdentityHashMap.access$1100(IdentityHashMap.this, entry.getKey(), entry.getValue());
        }
        
        @Override
        public boolean remove(Object o) {
            if (!(o instanceof Map.Entry)) {
                return false;
            }
            Map.Entry<Object, Object> entry = (Map.Entry)o;
            return IdentityHashMap.access$1200(IdentityHashMap.this, entry.getKey(), entry.getValue());
        }
        
        @Override
        public int size() {
            return IdentityHashMap.this.size;
        }
        
        @Override
        public void clear() {
            IdentityHashMap.this.clear();
        }
        
        @Override
        public boolean removeAll(Collection<?> collection) {
            Objects.requireNonNull(collection);
            boolean n = false;
            Iterator<Map.Entry<K, V>> iterator = this.iterator();
            while (iterator.hasNext()) {
                if (!collection.contains(iterator.next())) {
                    continue;
                }
                iterator.remove();
                n = true;
            }
            return n;
        }
        
        @Override
        public Object[] toArray() {
            return this.toArray(new Object[0]);
        }
        
        @Override
        public <T> T[] toArray(T[] array) {
            int modCount = IdentityHashMap.this.modCount;
            int size = this.size();
            if (array.length < size) {
                array = (Object[])(Object[])Array.newInstance(array.getClass().getComponentType(), size);
            }
            Object[] table = IdentityHashMap.this.table;
            int n = 0;
            for (int i = 0; i < table.length; i += 2) {
                Object o;
                if ((o = table[i]) != null) {
                    if (n >= size) {
                        throw new ConcurrentModificationException();
                    }
                    array[n++] = new AbstractMap.SimpleEntry(IdentityHashMap.unmaskNull(o), table[i + 1]);
                }
            }
            if (n < size || modCount != IdentityHashMap.this.modCount) {
                throw new ConcurrentModificationException();
            }
            if (n < array.length) {
                array[n] = null;
                return array;
            }
            return array;
        }
        
        @Override
        public Spliterator<Map.Entry<K, V>> spliterator() {
            return new IdentityHashMap.EntrySpliterator(IdentityHashMap.this, 0, -1, 0, 0);
        }
    }
    
    static class IdentityHashMapSpliterator<K, V>
    {
        final IdentityHashMap<K, V> map;
        int index;
        int fence;
        int est;
        int expectedModCount;
        
        IdentityHashMapSpliterator(IdentityHashMap<K, V> map, int index, int fence, int est, int expectedModCount) {
            this.map = map;
            this.index = index;
            this.fence = fence;
            this.est = est;
            this.expectedModCount = expectedModCount;
        }
        
        final int getFence() {
            int fence;
            if ((fence = this.fence) < 0) {
                this.est = this.map.size;
                this.expectedModCount = this.map.modCount;
                int length = this.map.table.length;
                this.fence = length;
                fence = length;
            }
            return fence;
        }
        
        public final long estimateSize() {
            this.getFence();
            return (long)this.est;
        }
    }
    
    static final class KeySpliterator<K, V> extends IdentityHashMap.IdentityHashMapSpliterator<K, V> implements Spliterator<K>
    {
        KeySpliterator(IdentityHashMap<K, V> identityHashMap, int n, int n2, int n3, int n4) {
            super(identityHashMap, n, n2, n3, n4);
        }
        
        @Override
        public IdentityHashMap.KeySpliterator<K, V> trySplit() {
            int fence = this.getFence();
            int index = this.index;
            int n = index + fence >>> 1 & 0xFFFFFFFE;
            if (index >= n) {
                return null;
            }
            IdentityHashMap<K, V> map;
            int n2;
            int index2;
            IdentityHashMap.KeySpliterator keySpliterator = new IdentityHashMap.KeySpliterator(map, n2, index2, this.est >>>= 1, this.expectedModCount);
            map = this.map;
            n2 = index;
            index2 = n;
            this.index = index2;
            return keySpliterator;
        }
        
        @Override
        public void forEachRemaining(Consumer<? super K> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            IdentityHashMap<Object, Object> map;
            Object[] table;
            int i;
            int fence;
            if ((map = this.map) != null && (table = map.table) != null && (i = this.index) >= 0 && (this.index = (fence = this.getFence())) <= table.length) {
                while (i < fence) {
                    Object o;
                    if ((o = table[i]) != null) {
                        consumer.accept(IdentityHashMap.unmaskNull(o));
                    }
                    i += 2;
                }
                if (map.modCount == this.expectedModCount) {
                    return;
                }
            }
            throw new ConcurrentModificationException();
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super K> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            Object[] table = this.map.table;
            while (this.index < this.getFence()) {
                Object o = table[this.index];
                this.index += 2;
                if (o != null) {
                    consumer.accept(IdentityHashMap.unmaskNull(o));
                    if (this.map.modCount != this.expectedModCount) {
                        throw new ConcurrentModificationException();
                    }
                    return true;
                }
                else {
                    continue;
                }
            }
            return false;
        }
        
        @Override
        public int characteristics() {
            return ((this.fence < 0 || this.est == this.map.size) ? 64 : 0) | 0x1;
        }
    }
    
    static final class ValueSpliterator<K, V> extends IdentityHashMap.IdentityHashMapSpliterator<K, V> implements Spliterator<V>
    {
        ValueSpliterator(IdentityHashMap<K, V> identityHashMap, int n, int n2, int n3, int n4) {
            super(identityHashMap, n, n2, n3, n4);
        }
        
        @Override
        public IdentityHashMap.ValueSpliterator<K, V> trySplit() {
            int fence = this.getFence();
            int index = this.index;
            int n = index + fence >>> 1 & 0xFFFFFFFE;
            if (index >= n) {
                return null;
            }
            IdentityHashMap<K, V> map;
            int n2;
            int index2;
            IdentityHashMap.ValueSpliterator valueSpliterator = new IdentityHashMap.ValueSpliterator(map, n2, index2, this.est >>>= 1, this.expectedModCount);
            map = this.map;
            n2 = index;
            index2 = n;
            this.index = index2;
            return valueSpliterator;
        }
        
        @Override
        public void forEachRemaining(Consumer<? super V> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            IdentityHashMap<Object, Object> map;
            Object[] table;
            int i;
            int fence;
            if ((map = this.map) != null && (table = map.table) != null && (i = this.index) >= 0 && (this.index = (fence = this.getFence())) <= table.length) {
                while (i < fence) {
                    if (table[i] != null) {
                        consumer.accept(table[i + 1]);
                    }
                    i += 2;
                }
                if (map.modCount == this.expectedModCount) {
                    return;
                }
            }
            throw new ConcurrentModificationException();
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super V> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            Object[] table = this.map.table;
            while (this.index < this.getFence()) {
                Object o = table[this.index];
                Object o2 = table[this.index + 1];
                this.index += 2;
                if (o != null) {
                    consumer.accept(o2);
                    if (this.map.modCount != this.expectedModCount) {
                        throw new ConcurrentModificationException();
                    }
                    return true;
                }
                else {
                    continue;
                }
            }
            return false;
        }
        
        @Override
        public int characteristics() {
            return (this.fence < 0 || this.est == this.map.size) ? 64 : 0;
        }
    }
    
    static final class EntrySpliterator<K, V> extends IdentityHashMap.IdentityHashMapSpliterator<K, V> implements Spliterator<Map.Entry<K, V>>
    {
        EntrySpliterator(IdentityHashMap<K, V> identityHashMap, int n, int n2, int n3, int n4) {
            super(identityHashMap, n, n2, n3, n4);
        }
        
        @Override
        public IdentityHashMap.EntrySpliterator<K, V> trySplit() {
            int fence = this.getFence();
            int index = this.index;
            int n = index + fence >>> 1 & 0xFFFFFFFE;
            if (index >= n) {
                return null;
            }
            IdentityHashMap<K, V> map;
            int n2;
            int index2;
            IdentityHashMap.EntrySpliterator entrySpliterator = new IdentityHashMap.EntrySpliterator(map, n2, index2, this.est >>>= 1, this.expectedModCount);
            map = this.map;
            n2 = index;
            index2 = n;
            this.index = index2;
            return entrySpliterator;
        }
        
        @Override
        public void forEachRemaining(Consumer<? super Map.Entry<K, V>> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            IdentityHashMap<Object, Object> map;
            Object[] table;
            int i;
            int fence;
            if ((map = this.map) != null && (table = map.table) != null && (i = this.index) >= 0 && (this.index = (fence = this.getFence())) <= table.length) {
                while (i < fence) {
                    Object o = table[i];
                    if (o != null) {
                        consumer.accept(new AbstractMap.SimpleImmutableEntry(IdentityHashMap.unmaskNull(o), table[i + 1]));
                    }
                    i += 2;
                }
                if (map.modCount == this.expectedModCount) {
                    return;
                }
            }
            throw new ConcurrentModificationException();
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super Map.Entry<K, V>> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            Object[] table = this.map.table;
            while (this.index < this.getFence()) {
                Object o = table[this.index];
                Object o2 = table[this.index + 1];
                this.index += 2;
                if (o != null) {
                    consumer.accept(new AbstractMap.SimpleImmutableEntry(IdentityHashMap.unmaskNull(o), o2));
                    if (this.map.modCount != this.expectedModCount) {
                        throw new ConcurrentModificationException();
                    }
                    return true;
                }
                else {
                    continue;
                }
            }
            return false;
        }
        
        @Override
        public int characteristics() {
            return ((this.fence < 0 || this.est == this.map.size) ? 64 : 0) | 0x1;
        }
    }
}
